package org.ninjasoft.rovr.ui;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.EmptyBorder;
import org.ninjasoft.rovr.net.*;
import org.ninjasoft.rovr.map.*;

public class RovrFrame extends JFrame {
    private boolean initialized = false;
    private Actions actions = new Actions();
    private Client client = null;
    private RovrKeyAdapter rovrKeyAdapter = new RovrKeyAdapter();
    private Mapper mapper = new Mapper();
    private MapFrame mapFrame = new MapFrame(mapper);
    private JComboBox speedCombo = new JComboBox(new String[] {"x1", "x2", "x3", "x4", "x5"});
	private JTextArea instructionText;
	private JTextField statusText;
	private JCheckBox magnetCheckbox = new JCheckBox("Magnet");

    private static final String INSTRUCTIONS = 
    	"Movement is done using the keys 'A' and 'Z' on the left hand for up/down and " +
    	"',' and '.' on the right hand for left/right.  Arrow keys also work.  " +
    	"Use numbers 1 through 5 to adjust speed.  " + 
    	"Press [SPACE] to stop all movement.";
    
    public void initialize() {
        initializeGui();
        initializeEvents();
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    }
    
    private void initializeGui() {
    	JPanel buttonPanel;
    	JButton button;
    	JPanel centerPanel = new JPanel();;
        if (initialized)
            return;
        initialized = true;
        this.setSize(300, 200);
        Dimension windowSize = this.getSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        this.setLocation(screenSize.width/2 - windowSize.width/2, screenSize.height/2 - windowSize.height/2);
        Container pane = this.getContentPane();
        pane.setLayout(new BorderLayout());
        centerPanel.setLayout(new BorderLayout());
        instructionText = new JTextArea();
        instructionText.setLineWrap(true);
        instructionText.setText(INSTRUCTIONS);
        instructionText.setEditable(false);
        instructionText.setBorder(new EmptyBorder(0,0,0,0));
        instructionText.setForeground(Color.BLACK);
        instructionText.setBackground(pane.getBackground());
        instructionText.setMinimumSize(new Dimension(100, 100));
        instructionText.setWrapStyleWord(true);
        centerPanel.add(new JScrollPane(instructionText), BorderLayout.NORTH);
        pane.add(centerPanel, BorderLayout.CENTER);
        statusText = new JTextField();
        statusText.setEditable(false);
        statusText.setBorder(new EmptyBorder(0,0,0,0));
        statusText.setForeground(Color.BLACK);
        statusText.setBackground(pane.getBackground());
        pane.add(new JScrollPane(statusText), BorderLayout.SOUTH);
        pane.add(mapFrame, BorderLayout.CENTER);
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        
        button = new JButton("Reset Map");
        button.setActionCommand("resetmap");
        button.addActionListener(actions);
        button.addKeyListener(rovrKeyAdapter);
        buttonPanel.add(button);
        
        buttonPanel.add(new JLabel("Speed:"));
        buttonPanel.add(this.speedCombo);
        this.speedCombo.setSize(50, this.speedCombo.getHeight());
        
        buttonPanel.add(magnetCheckbox);
        
        pane.add(buttonPanel, BorderLayout.NORTH);
    }
    
    private void initializeEvents() {
        this.client = new Client();
        client.addMovementListener(new MovementListenerStdout());
        client.addMovementListener(new MovementListenerTextField(statusText));
        client.addMovementListener(mapper);
        mapper.addMapChangeListener(mapFrame);
        client.addMovementListener(new StopListener());
        this.addKeyListener(rovrKeyAdapter);
        instructionText.addKeyListener(rovrKeyAdapter);
        statusText.addKeyListener(rovrKeyAdapter);
        speedCombo.addKeyListener(rovrKeyAdapter);
        magnetCheckbox.addKeyListener(rovrKeyAdapter);
        mapFrame.addKeyListener(rovrKeyAdapter);
        speedCombo.setActionCommand("speed");
        speedCombo.addActionListener(actions);
        magnetCheckbox.setSelected(true);
        try {
        	client.connect();
        }catch(Throwable t) {
        	JOptionPane.showMessageDialog(this, "Could not connect to maze program", "Fatal Error", JOptionPane.ERROR_MESSAGE);
        	System.exit(0);
        }
    }
    
    public class Actions implements ActionListener {
		public void actionPerformed(ActionEvent e) {
            String command = e.getActionCommand();
            command = command == null ? "" : command;
            if (command.equals("showmap")) {
            	mapFrame.setVisible(true);
            	instructionText.grabFocus();
            } else if (command.equals("resetmap")) {
            	mapper.reset();
            } else if (command.equals("speed")) {
            	client.setMoveFactor(speedCombo.getSelectedIndex() + 1);
            }
        }
    }

    public void dispose() {
        // TODO: Save settings
        //super.dispose();
        System.exit(0);
    }
    
    public void setVisible(boolean b) {
        initialize();
        super.setVisible(b);
    }
    
    void movementKey(int key) {
    	try {
	    	switch(key) {
	    	case 'a':
	    		client.sendCommand(Client.COMMAND_UP);
	    		mapper.direction = Client.COMMAND_UP;
	    		break;
	    	case 'z':
	    		client.sendCommand(Client.COMMAND_DOWN);
	    		mapper.direction = Client.COMMAND_DOWN;
	    		break;
	    	case ',':
	    		client.sendCommand(Client.COMMAND_LEFT);
	    		mapper.direction = Client.COMMAND_LEFT;
	    		break;
	    	case '.':
	    		client.sendCommand(Client.COMMAND_RIGHT);
	    		mapper.direction = Client.COMMAND_RIGHT;
	    		break;
	    	case ' ':
	    		client.sendCommand(Client.COMMAND_STOP);
	    		mapper.direction = Client.COMMAND_STOP;
	    		break;
	    	case '1':
	    	case '2':
	    	case '3':
	    	case '4':
	    	case '5':
	    		speedCombo.setSelectedIndex(key - '1');
	    		client.setMoveFactor(speedCombo.getSelectedIndex() + 1);
	    		break;
	    	}
    	}catch(Exception e) {
    		System.out.println("Error sending command!");
    		// TODO: better handle exception
    	}
    }
    
	public static void main(String[] args) {
        new RovrFrame().setVisible(true);
	}
	
	public class StopListener implements MovementListener {
		public void movementResponse(int code) {
			try{
				if (magnetCheckbox.isSelected()) {
					client.sendCommand(Client.COMMAND_STOP);
					mapper.direction = Client.COMMAND_STOP;
				}
			}catch(Exception e) {}
		}
	}
	
	public class RovrKeyAdapter extends KeyAdapter {
		public void keyPressed(KeyEvent e) {
			int rovrKey = 0;
			switch(e.getKeyChar()) {
			case 'A':
			case 'a':
				rovrKey = 'a';
				break;
			case 'Z':
			case 'z':
				rovrKey = 'z';
				break;
			case ',':
				rovrKey = ',';
				break;
			case '.':
				rovrKey = '.';
				break;
			case ' ':
				rovrKey = ' ';
				break;
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
				rovrKey = e.getKeyChar();
				break;
			}
			switch(e.getKeyCode()) {
			case KeyEvent.VK_KP_UP:
			case KeyEvent.VK_UP:
			case KeyEvent.VK_NUMPAD8:
				rovrKey = 'a';
				break;
			case KeyEvent.VK_KP_DOWN:
			case KeyEvent.VK_DOWN:
			case KeyEvent.VK_NUMPAD2:
				rovrKey = 'z';
				break;
			case KeyEvent.VK_KP_LEFT:
			case KeyEvent.VK_LEFT:
			case KeyEvent.VK_NUMPAD4:
				rovrKey = ',';
				break;
			case KeyEvent.VK_KP_RIGHT:
			case KeyEvent.VK_RIGHT:
			case KeyEvent.VK_NUMPAD7:
				rovrKey = '.';
				break;
			case KeyEvent.VK_NUMPAD5:
				rovrKey = ' ';
				break;
			}
			if (rovrKey != 0)
				movementKey(rovrKey);
		}
	}
}
